"""
Interface for Pmod ACL2 module.
"""

# TODO
# FIFO setting
# Active and inactive detection


import spidev
import RPi.GPIO as GPIO
import time
CAP = 'SPI'
PHY = '2x6'


class PmodACL2:
    
    def __init__(self, DSPMod12):
        self.selectedRange = 1
        self.port = DSPMod12
        self.cs = self.port.pin1
        self.nc = self.port.pin2
        self.miso = self.port.pin3
        self.sclk = self.port.pin4
        
        self.int2 = self.port.pin7
        self.int1 = self.port.pin8
        GPIO.setmode(GPIO.BCM)  #
        GPIO.setup(self.int1,GPIO.IN)   #
        GPIO.setup(self.int2,GPIO.IN)   #

        # Register setting
        # ADXL362 range options
        self.SENSOR_RANGE_2G = 0 # +/-2 g
        self.SENSOR_RANGE_4G = 1 # +/-4 g
        self.SENSOR_RANGE_8G = 2 # +/-8 g
        # ADXL362 range options
        self.SENSOR_ODR_12_5_HZ= 0 # 12.5 Hz
        self.SENSOR_ODR_25_HZ  = 1 # 25 Hz
        self.SENSOR_ODR_50_HZ  = 2 # 50 Hz
        self.SENSOR_ODR_100_HZ = 3 # 100 Hz
        self.SENSOR_ODR_200_HZ = 4 # 200 Hz
        self.SENSOR_ODR_400_HZ = 5 # 400 Hz
        
        self.spi = spidev.SpiDev()
        self.__startSPI()

        self.softwareReset()
        # self.__spiWrite( 0x1F, 0x52)         # Software Reset
        time.sleep(0.01)
        self.setRange(1)                     # 4g
        self.setOutputRate(3)                # 100Hz
        self.setPowerMode(0x02)              # Measurement mode
       
    def __startSPI(self):
        if self.cs == 7: #CE1
            self.spi.open(0,1)
        elif self.cs == 8: #CE0
            self.spi.open(0,0)
        else:
            #throw exception here
            return
        self.spi.max_speed_hz = 100000
        self.spi.mode = 0b00
        
    def __stopSPI(self):
        self.spi.close()

    def __spiRead(self, Address):
        response = self.spi.xfer2([0x0B, Address, 0x00])
        return response[-1]

    def __spiRead16(self, Address):
        resp = self.spi.xfer2([0x0B, Address, 0x00, 0x00])
        # w = resp[3] 
        # w <<= 8
        # w |= resp[2]
        val_l = resp[2]
        val_h = resp[3] << 8
        w = (val_l + val_h) 

        return w

    def __spiWrite(self,address,value):
        self.spi.xfer2([0x0A, address, value])

    def __spiWrite16(self, address, value):
        highValue = value >> 8
        lowValue  = value & 0xFF
        self.spi.xfer2([0x0A, address, lowValue, highValue])
    
    def __unsigned2int(self, val):
        if val&(1<<15) != 0:
            val = val - (1<<16)
        return val

    def cleanup(self):
        self.__stopSPI()
        GPIO.cleanup()

    def setRange(self,newRange):
        # Range options: +/- 2,4,8 [unit: g]
        # 0: 2g
        # 1: 4g
        # 2: 8g
        if (newRange == self.SENSOR_RANGE_2G) or (newRange == self.SENSOR_RANGE_4G) or (newRange == self.SENSOR_RANGE_8G):
            oldFilterCtl = self.__spiRead(0x2C)
            time.sleep(0.01)
            newFilterCtl = oldFilterCtl & ~((0x3) << 6)
            newFilterCtl = newFilterCtl | (((newRange) & 0x03) << 6)
            self.__spiWrite(0x2C, newFilterCtl)
            time.sleep(0.01)
            self.selectedRange = (1 << newRange) * 2
            # print(self.selectedRange)
            return True
        else:
            return False

    def setOutputRate(self,outRate):
        # Rate Options:
        # 0: 12.5Hz,
        # 1: 25Hz,
        # 2: 50Hz,
        # 3: 100Hz,
        # 4: 200Hz,
        # 5: 400Hz
        if(outRate >=0)and(outRate <= 5):
            oldFilterCtl = self.__spiRead(0x2C) # ADXL362_REG_FILTER_CTL = 0x2C
            time.sleep(0.01)
            newFilterCtl = oldFilterCtl & ~0x7
            newFilterCtl = newFilterCtl | (0x7 & outRate)
            self.__spiWrite(0x2C, newFilterCtl)
            time.sleep(0.01)
            return True
        else:
            return False

    def setPowerMode(self,PowerMode):
        # Power Mode:
        # 0x0: standby
        # 0x2: measurement
        # self.spi.cshigh = True
        if (PowerMode == 0x00) or (PowerMode == 0x02):
            oldPowerCtl = self.__spiRead(0x2D) # ADXL362_REG_POWER_CTL = 0x2D
            time.sleep(0.01)
            newPowerCtl = oldPowerCtl & 0xFC
            newPowerCtl = newPowerCtl | (PowerMode & 0x03)
            self.__spiWrite(0x2D, newPowerCtl)
            # print(hex(newPowerCtl))
            # self.spi.cshigh = False
            time.sleep(0.01)
            return True
        else:
            return False

    def softwareReset(self):
        self.__spiWrite(0x1F, 0x52)         # Software Reset

    def getX(self):
        x = self.__spiRead16(0x0E)
        x = self.__unsigned2int(x)
        return x

    def getY(self):
        y = self.__spiRead16(0x10)
        y = self.__unsigned2int(y)
        return y

    def getZ(self):
        z = self.__spiRead16(0x12)
        z = self.__unsigned2int(z)
        return z

    def getTemperature(self):
        temp = self.__spiRead16(0x14)
        temp = self.__unsigned2int(temp)
        return temp

    def getRawXYZT(self):
        # return raw data
        x = self.getX()
        y = self.getY()
        z = self.getZ()
        t = self.getTemperature()
        return x, y, z, t

    def getXYZT(self):
        # return data in phisical unit
        x = self.getX()/(1000.0/(self.selectedRange/2.0))
        y = self.getY()/(1000.0/(self.selectedRange/2.0))
        z = self.getZ()/(1000.0/(self.selectedRange/2.0))
        t = self.getTemperature()*0.065
        return x, y, z, t

    def getDeviceID(self):
        # Get device ID. If device is connected properly, 0xAD will be returnd.
        DEVID_AD = self.__spiRead(0x00)
        return DEVID_AD

    def getPowerMode(self):
        # Get Power Mode.
        PowerMode = self.__spiRead(0x2d)
        return PowerMode
    
    def getSensorStatus(self):
        # Get status register (0x0B)
        StatusReg = self.__spiRead(0x0B)
        return StatusReg



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for PmodHB3 module.
"""

import RPi.GPIO as gpio

CAP = 'GPIO'
PHY = '1x6'
    
STOPPED = 0
FORWARD = 1
REVERSE = -1

class PmodHB3:

    def __init__(self, DSPMod6):
        
        self.port = DSPMod6
        
        self.dir = self.port.pin1
        self.en = self.port.pin2
        self.sA = self.port.pin3
        self.sB = self.port.pin4
        
        gpio.setmode(gpio.BCM)
        gpio.setup(self.dir, gpio.OUT)
        gpio.output(self.dir, gpio.LOW)
        gpio.setup(self.en, gpio.OUT)
        
        gpio.setup(self.sA, gpio.IN, pull_up_down=gpio.PUD_UP)
        gpio.setup(self.sB, gpio.IN, pull_up_down=gpio.PUD_UP)
        
        self.enable = gpio.PWM(self.en,100)
        self.enable.stop()
        
        self.direction = STOPPED 
        
    def forward(self,duty):
        if self.direction == STOPPED or self.direction == FORWARD:
            gpio.output(self.dir, gpio.HIGH)
            self.enable.start(0)
            
        else:
            self.enable.ChangeDutyCycle(0)
            self.enable.stop()
            gpio.output(self.dir, gpio.HIGH)
            self.enable.start(0)
            
        self.enable.ChangeDutyCycle(duty)    
        self.direction = FORWARD
            
    def reverse(self,duty):
        if self.direction == STOPPED or self.direction == REVERSE:
            gpio.output(self.dir, gpio.LOW)
            self.enable.start(0)
            
        else:
            self.enable.ChangeDutyCycle(0)
            self.enable.stop()
            gpio.output(self.dir, gpio.LOW)
            self.enable.start(0)
            
        self.enable.ChangeDutyCycle(duty)    
        self.direction = REVERSE
        
    def stop(self):
        self.enable.ChangeDutyCycle(0)
        self.enable.stop()
        self.direction = STOPPED
        
    def changeFrequency(self,freq):
        self.enable.ChangeFrequency(freq)
        
    def cleanup(self):
        gpio.cleanup();
        



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for PmodOLEDrgb module (ssd1331).

.. note::
   Depends on luma.oled and luma.core.
"""

import RPi.GPIO as gpio
from luma.core.render import canvas
from luma.core.sprite_system import framerate_regulator

from luma.core.interface.serial import spi
from luma.oled.device import ssd1331

CAP = 'SPI'
PHY = '2x6'

class PmodOLEDrgb:

    def __init__(self, DSPMod12):
        self.port = DSPMod12
        
        self.cs = self.port.pin1
        self.mosi = self.port.pin2
        self.nc = self.port.pin3
        self.sclk = self.port.pin4
        
        self.dc = self.port.pin7
        self.rst = self.port.pin8
        self.vccen = self.port.pin9
        self.pmoden = self.port.pin10
        self.powerf = True
        
        #
        if self.cs == 7: #CE1
            self.serial = spi(device=1, port=0, gpio_DC=self.dc, gpio_RST=self.rst)
        elif self.cs == 8: #CE0
            self.serial = spi(device=0, port=0, gpio_DC=self.dc, gpio_RST=self.rst)
        else:
            #throw exception
            pass

        # Setup GPIO.
        gpio.setmode(gpio.BCM)
        gpio.setup(self.vccen, gpio.OUT)
        gpio.output(self.vccen, gpio.HIGH)
        gpio.setup(self.pmoden, gpio.OUT)
        gpio.output(self.pmoden, gpio.HIGH)
        
        self.device = ssd1331(self.serial)
    
    def powerOff(self):
        gpio.output(self.vccen, gpio.LOW)
        gpio.output(self.pmoden, gpio.LOW)
        self.powerf = True
        
    def powerOn(self):
        gpio.output(self.vccen, gpio.HIGH)
        gpio.output(self.pmoden, gpio.HIGH)
        self.powerf = False
    
    def cleanup(self):
        self.powerOff
        #gpio.cleanup()
        self.device.cleanup()
    
    def getDevice(self):
        return self.device



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

class Error(Exception):
    """
    Base class for exceptions in this library.
    """
    pass

class portCapabilitySupport(Error):
    """
    Exception raised when a port does not support the module type.
    """

class portCapabilityConflict(Error):
    """
    Exception raised when the port is already using shared GPIO pins.
    """
  
class portInUse(Error):
    """
    Exception raised when the port is already assigned.
    """  

class incorrectPortName(Error):
    """
    Exception raised when the port name given does not exist in the port map.
    """
    
class incorrectModuleName(Error):
    """
    Exception raised when the module name given does not exist in the module map.
    """



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for PmodAD1 module (AD7476A).

.. note::
   Only a single channel (A1) is supported at present due to the way that SPI is configured.
"""

import spidev
CAP = 'SPI'
PHY = '1x6'

class PmodAD1:
    
    def __init__(self, DSPMod6):
        
        self.port = DSPMod6
        self.cs = self.port.pin1
        self.d0 = self.port.pin2
        self.d1 = self.port.pin3 #miso
        self.clk = self.port.pin4
        
        self.spi = spidev.SpiDev()
        self.__startSPI()
       
    def __startSPI(self):
        if self.cs == 7: #CE1
            self.spi.open(0,1)
        elif self.cs == 8: #CE0
            self.spi.open(0,0)
        else:
            #throw exception here
            return
        self.spi.max_speed_hz = 100000
        self.spi.mode = 0b00
        
    def __stopSPI(self):
        self.spi.close()   
    
    def __spiRead16(self):
            resp = self.spi.xfer2([0x00, 0x00])
            w = resp[0] 
            w <<= 8
            w |= resp[1]
            return w
    
    def readA1(self):
            return self.__spiRead16()
    
    def readA1Volts(self):
            w = self.__spiRead16()
            
            refV = 3.3
            lsb =  refV/4096
            volts= w*lsb
            
            return volts
 
    def cleanup(self):
            self.__stopSPI()




# -*- coding: utf-8 -*-
# Copyright (c) 2019 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for Pmod_LS1
"""

import RPi.GPIO as GPIO
CAP = 'GPIO'
PHY = '1x6'

class PmodLS1:
    def __init__(self,DSPmod6):
        # GPIO setup
        self.port = DSPmod6
        self.SNS1 = self.port.pin1
        self.SNS2 = self.port.pin2
        self.SNS3 = self.port.pin3
        self.SNS4 = self.port.pin4
        GPIO.setmode(GPIO.BCM)  #
        GPIO.setup(self.SNS1,GPIO.IN)   #
        GPIO.setup(self.SNS2,GPIO.IN)   #
        GPIO.setup(self.SNS3,GPIO.IN)   #
        GPIO.setup(self.SNS4,GPIO.IN)   #

    def cleanup(self):
        GPIO.cleanup()

    def GetSensorPin(self,Sensor):
        # returns phisical GPIO Pin assignment
        if Sensor == 1:
            return self.SNS1
        elif Sensor == 2:
            return self.SNS2
        elif Sensor == 3:
            return self.SNS3
        elif Sensor == 4:
            return self.SNS4
        else:
            return None
    def GetStatus(self,SNS):
        # return sensor status
        sensorPin = self.GetSensorPin(SNS)
        return GPIO.input(sensorPin)
    def GetAllStatus(self):
        # return all sensor status
        return self.GetStatus(1)<<0 | self.GetStatus(2)<<1 | self.GetStatus(3)<<2 | self.GetStatus(4)<<3



# -*- coding: utf-8 -*-
# Copyright (c) 2019 RS Components Ltd
# SPDX-License-Identifier: MIT License

# MicropyGPS - a GPS NMEA sentence parser for Micropython/Python 3.X
# Copyright (c) 2017 Michael Calvin McCoy (calvin.mccoy@protonmail.com)
# https://github.com/inmcm/micropyGPS

"""
Interface for PmodGPS
"""


from math import floor, modf
# Import utime or time for fix time handling
try:
    # Assume running on MicroPython
    import utime
except ImportError:
    # Otherwise default to time module for non-embedded implementations
    # Should still support millisecond resolution.
    import time

# Import serial for Pmod
import serial
CAP = 'UART'
PHY = '1x6'

class PmodGPS:
    
    def __init__(self, DSPMod6):
        
        self.port = DSPMod6
        self.ser = serial.Serial('/dev/serial0',9600,timeout=None)
        self.pygps = MicropyGPS()
        
    def cleanup(self):
        self.ser.close()

    def getGPSLine(self):
        gpsLine = "null"
        while(gpsLine[0] != '$'): # discard trash data
            gpsLine = self.ser.readline().strip().decode('utf-8')
        return gpsLine
        # return self.ser.readline().strip().decode('utf-8')
    def gpsUpdate(self):
        while self.ser.in_waiting > 1:
            line = self.getGPSLine()
            for x in line:
                self.pygps.update(x)

    def isFixed(self):
        # return: 1: no-fix, 2: 2D-fixed, 3: 3D-fixed
        return self.pygps.fix_type
    def getLatitude(self):
        # return: Latitude[(deg),(min),E or W]
        return self.pygps.latitude
    def getLongitude(self):
        # return: Longitude[(deg),(min),N or S]
        return self.pygps.longitude
    def getDate(self):
        # return: Date
        return self.pygps.date
    def getTime(self):
        # return: Time (UTC)
        return self.pygps.timestamp
    def getAltitude(self):
        # return: Altitude[m]
        return self.pygps.altitude
    def getSpeedKM(self):
        # return: Absolute speed[km/h]
        return self.pygps.speed[2]
    """
    def getSpeedKnot(self):
        # return: Absolute speed[Knot]
        # SI unit is STRONGLY recommended
        return self.pygps.speed[0]
    def getSpeedMPH(self):
        # return: Absolute speed[Mile/hour]
        # SI unit is STRONGLY recommended
        return self.pygps.speed[1]
    """
    def getNumSats(self):
        # return: Number of satellites
        return self.pygps.satellites_in_use
    def getSatelliteInfo(self):
        # retuen: Satellite number which used in positioning
        return self.pygps.satellites_used
    def getSatelliteData(self):
        # return: (Elevation, Azimuth, SNR (if available))
        return self.pygps.satellite_data
    def getVisibleSatellite(self):
        # return satellite number of visible ones
        return self.pygps.satellites_visible()
    def getPDOP(self):
        # return positioning precision
        return self.pygps.pdop
    def getHeading(self):
        # return heading direction[deg]
        return self.pygps.course
    
    def getGPSPosData(self):
        day,month,year = self.getDate()
        hour,minute,sec = self.getTime()
        LatDeg,LatMin,NS = self.getLatitude()
        LogDeg,LogMin,EW = self.getLongitude()
        # speed = self.getSpeedKM()
        # heading = self.getHeading()
        PDOP = self.getPDOP()

        LatDeg = LatDeg + LatMin/60.0
        LogDeg = LogDeg + LogMin/60.0
        year = year + 2000
        if NS == 'S':
            LatDeg = -LatDeg
        if EW == 'W':
            LogDeg = -LogDeg

        return day,month,year,hour,minute,sec,LatDeg,LogDeg,PDOP


# Class "MicropyGPS" is forked from MicropyGPS
# https://github.com/inmcm/micropyGPS

class MicropyGPS:
    """GPS NMEA Sentence Parser. Creates object that stores all relevant GPS data and statistics.
    Parses sentences one character at a time using update(). """

    # Max Number of Characters a valid sentence can be (based on GGA sentence)
    SENTENCE_LIMIT = 90
    __HEMISPHERES = ('N', 'S', 'E', 'W')
    __NO_FIX = 1
    __FIX_2D = 2
    __FIX_3D = 3
    __DIRECTIONS = ('N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 'SE', 'SSE', 'S', 'SSW', 'SW', 'WSW', 'W',
                    'WNW', 'NW', 'NNW')
    __MONTHS = ('January', 'February', 'March', 'April', 'May',
                'June', 'July', 'August', 'September', 'October',
                'November', 'December')

    def __init__(self, local_offset=0, location_formatting='ddm'):
        """
        Setup GPS Object Status Flags, Internal Data Registers, etc
            local_offset (int): Timzone Difference to UTC
            location_formatting (str): Style For Presenting Longitude/Latitude:
                                       Decimal Degree Minute (ddm) - 40° 26.767′ N
                                       Degrees Minutes Seconds (dms) - 40° 26′ 46″ N
                                       Decimal Degrees (dd) - 40.446° N
        """

        #####################
        # Object Status Flags
        self.sentence_active = False
        self.active_segment = 0
        self.process_crc = False
        self.gps_segments = []
        self.crc_xor = 0
        self.char_count = 0
        self.fix_time = 0

        #####################
        # Sentence Statistics
        self.crc_fails = 0
        self.clean_sentences = 0
        self.parsed_sentences = 0

        #####################
        # Logging Related
        self.log_handle = None
        self.log_en = False

        #####################
        # Data From Sentences
        # Time
        self.timestamp = [0, 0, 0]
        self.date = [0, 0, 0]
        self.local_offset = local_offset

        # Position/Motion
        self._latitude = [0, 0.0, 'N']
        self._longitude = [0, 0.0, 'W']
        self.coord_format = location_formatting
        self.speed = [0.0, 0.0, 0.0]
        self.course = 0.0
        self.altitude = 0.0
        self.geoid_height = 0.0

        # GPS Info
        self.satellites_in_view = 0
        self.satellites_in_use = 0
        self.satellites_used = []
        self.last_sv_sentence = 0
        self.total_sv_sentences = 0
        self.satellite_data = dict()
        self.hdop = 0.0
        self.pdop = 0.0
        self.vdop = 0.0
        self.valid = False
        self.fix_stat = 0
        self.fix_type = 1

    ########################################
    # Coordinates Translation Functions
    ########################################
    @property
    def latitude(self):
        """Format Latitude Data Correctly"""
        if self.coord_format == 'dd':
            decimal_degrees = self._latitude[0] + (self._latitude[1] / 60)
            return [decimal_degrees, self._latitude[2]]
        elif self.coord_format == 'dms':
            minute_parts = modf(self._latitude[1])
            seconds = round(minute_parts[0] * 60)
            return [self._latitude[0], int(minute_parts[1]), seconds, self._latitude[2]]
        else:
            return self._latitude

    @property
    def longitude(self):
        """Format Longitude Data Correctly"""
        if self.coord_format == 'dd':
            decimal_degrees = self._longitude[0] + (self._longitude[1] / 60)
            return [decimal_degrees, self._longitude[2]]
        elif self.coord_format == 'dms':
            minute_parts = modf(self._longitude[1])
            seconds = round(minute_parts[0] * 60)
            return [self._longitude[0], int(minute_parts[1]), seconds, self._longitude[2]]
        else:
            return self._longitude

    ########################################
    # Logging Related Functions
    ########################################
    def start_logging(self, target_file, mode="append"):
        """
        Create GPS data log object
        """
        # Set Write Mode Overwrite or Append
        mode_code = 'w' if mode == 'new' else 'a'

        try:
            self.log_handle = open(target_file, mode_code)
        except AttributeError:
            print("Invalid FileName")
            return False

        self.log_en = True
        return True

    def stop_logging(self):
        """
        Closes the log file handler and disables further logging
        """
        try:
            self.log_handle.close()
        except AttributeError:
            print("Invalid Handle")
            return False

        self.log_en = False
        return True

    def write_log(self, log_string):
        """Attempts to write the last valid NMEA sentence character to the active file handler
        """
        try:
            self.log_handle.write(log_string)
        except TypeError:
            return False
        return True

    ########################################
    # Sentence Parsers
    ########################################
    def gprmc(self):
        """Parse Recommended Minimum Specific GPS/Transit data (RMC)Sentence.
        Updates UTC timestamp, latitude, longitude, Course, Speed, Date, and fix status
        """

        # UTC Timestamp
        try:
            utc_string = self.gps_segments[1]

            if utc_string:  # Possible timestamp found
                hours = (int(utc_string[0:2]) + self.local_offset) % 24
                minutes = int(utc_string[2:4])
                seconds = float(utc_string[4:])
                self.timestamp = (hours, minutes, seconds)
            else:  # No Time stamp yet
                self.timestamp = (0, 0, 0)

        except ValueError:  # Bad Timestamp value present
            return False

        # Date stamp
        try:
            date_string = self.gps_segments[9]

            # Date string printer function assumes to be year >=2000,
            # date_string() must be supplied with the correct century argument to display correctly
            if date_string:  # Possible date stamp found
                day = int(date_string[0:2])
                month = int(date_string[2:4])
                year = int(date_string[4:6])
                self.date = (day, month, year)
            else:  # No Date stamp yet
                self.date = (0, 0, 0)

        except ValueError:  # Bad Date stamp value present
            return False

        # Check Receiver Data Valid Flag
        if self.gps_segments[2] == 'A':  # Data from Receiver is Valid/Has Fix

            # Longitude / Latitude
            try:
                # Latitude
                l_string = self.gps_segments[3]
                lat_degs = int(l_string[0:2])
                lat_mins = float(l_string[2:])
                lat_hemi = self.gps_segments[4]

                # Longitude
                l_string = self.gps_segments[5]
                lon_degs = int(l_string[0:3])
                lon_mins = float(l_string[3:])
                lon_hemi = self.gps_segments[6]
            except ValueError:
                return False

            if lat_hemi not in self.__HEMISPHERES:
                return False

            if lon_hemi not in self.__HEMISPHERES:
                return False

            # Speed
            try:
                spd_knt = float(self.gps_segments[7])
            except ValueError:
                return False

            # Course
            try:
                if self.gps_segments[8]:
                    course = float(self.gps_segments[8])
                else:
                    course = 0.0
            except ValueError:
                return False

            # TODO - Add Magnetic Variation

            # Update Object Data
            self._latitude = [lat_degs, lat_mins, lat_hemi]
            self._longitude = [lon_degs, lon_mins, lon_hemi]
            # Include mph and hm/h
            self.speed = [spd_knt, spd_knt * 1.151, spd_knt * 1.852]
            self.course = course
            self.valid = True

            # Update Last Fix Time
            self.new_fix_time()

        else:  # Clear Position Data if Sentence is 'Invalid'
            self._latitude = [0, 0.0, 'N']
            self._longitude = [0, 0.0, 'W']
            self.speed = [0.0, 0.0, 0.0]
            self.course = 0.0
            self.valid = False

        return True

    def gpgll(self):
        """Parse Geographic Latitude and Longitude (GLL)Sentence. Updates UTC timestamp, latitude,
        longitude, and fix status"""

        # UTC Timestamp
        try:
            utc_string = self.gps_segments[5]

            if utc_string:  # Possible timestamp found
                hours = (int(utc_string[0:2]) + self.local_offset) % 24
                minutes = int(utc_string[2:4])
                seconds = float(utc_string[4:])
                self.timestamp = (hours, minutes, seconds)
            else:  # No Time stamp yet
                self.timestamp = (0, 0, 0)

        except ValueError:  # Bad Timestamp value present
            return False

        # Check Receiver Data Valid Flag
        if self.gps_segments[6] == 'A':  # Data from Receiver is Valid/Has Fix

            # Longitude / Latitude
            try:
                # Latitude
                l_string = self.gps_segments[1]
                lat_degs = int(l_string[0:2])
                lat_mins = float(l_string[2:])
                lat_hemi = self.gps_segments[2]

                # Longitude
                l_string = self.gps_segments[3]
                lon_degs = int(l_string[0:3])
                lon_mins = float(l_string[3:])
                lon_hemi = self.gps_segments[4]
            except ValueError:
                return False

            if lat_hemi not in self.__HEMISPHERES:
                return False

            if lon_hemi not in self.__HEMISPHERES:
                return False

            # Update Object Data
            self._latitude = [lat_degs, lat_mins, lat_hemi]
            self._longitude = [lon_degs, lon_mins, lon_hemi]
            self.valid = True

            # Update Last Fix Time
            self.new_fix_time()

        else:  # Clear Position Data if Sentence is 'Invalid'
            self._latitude = [0, 0.0, 'N']
            self._longitude = [0, 0.0, 'W']
            self.valid = False

        return True

    def gpvtg(self):
        """Parse Track Made Good and Ground Speed (VTG) Sentence. Updates speed and course"""
        try:
            course = float(self.gps_segments[1])
            spd_knt = float(self.gps_segments[5])
        except ValueError:
            return False

        # Include mph and km/h
        self.speed = (spd_knt, spd_knt * 1.151, spd_knt * 1.852)
        self.course = course
        return True

    def gpgga(self):
        """Parse Global Positioning System Fix Data (GGA) Sentence. Updates UTC timestamp, latitude, longitude,
        fix status, satellites in use, Horizontal Dilution of Precision (HDOP), altitude, geoid height and fix status"""

        try:
            # UTC Timestamp
            utc_string = self.gps_segments[1]

            # Skip timestamp if receiver doesn't have on yet
            if utc_string:
                hours = (int(utc_string[0:2]) + self.local_offset) % 24
                minutes = int(utc_string[2:4])
                seconds = float(utc_string[4:])
            else:
                hours = 0
                minutes = 0
                seconds = 0.0

            # Number of Satellites in Use
            satellites_in_use = int(self.gps_segments[7])

            # Get Fix Status
            fix_stat = int(self.gps_segments[6])

        except (ValueError, IndexError):
            return False

        try:
            # Horizontal Dilution of Precision
            hdop = float(self.gps_segments[8])
        except (ValueError, IndexError):
            hdop = 0.0

        # Process Location and Speed Data if Fix is GOOD
        if fix_stat:

            # Longitude / Latitude
            try:
                # Latitude
                l_string = self.gps_segments[2]
                lat_degs = int(l_string[0:2])
                lat_mins = float(l_string[2:])
                lat_hemi = self.gps_segments[3]

                # Longitude
                l_string = self.gps_segments[4]
                lon_degs = int(l_string[0:3])
                lon_mins = float(l_string[3:])
                lon_hemi = self.gps_segments[5]
            except ValueError:
                return False

            if lat_hemi not in self.__HEMISPHERES:
                return False

            if lon_hemi not in self.__HEMISPHERES:
                return False

            # Altitude / Height Above Geoid
            try:
                altitude = float(self.gps_segments[9])
                geoid_height = float(self.gps_segments[11])
            except ValueError:
                altitude = 0
                geoid_height = 0

            # Update Object Data
            self._latitude = [lat_degs, lat_mins, lat_hemi]
            self._longitude = [lon_degs, lon_mins, lon_hemi]
            self.altitude = altitude
            self.geoid_height = geoid_height

        # Update Object Data
        self.timestamp = [hours, minutes, seconds]
        self.satellites_in_use = satellites_in_use
        self.hdop = hdop
        self.fix_stat = fix_stat

        # If Fix is GOOD, update fix timestamp
        if fix_stat:
            self.new_fix_time()

        return True

    def gpgsa(self):
        """Parse GNSS DOP and Active Satellites (GSA) sentence. Updates GPS fix type, list of satellites used in
        fix calculation, Position Dilution of Precision (PDOP), Horizontal Dilution of Precision (HDOP), Vertical
        Dilution of Precision, and fix status"""

        # Fix Type (None,2D or 3D)
        try:
            fix_type = int(self.gps_segments[2])
        except ValueError:
            return False

        # Read All (up to 12) Available PRN Satellite Numbers
        sats_used = []
        for sats in range(12):
            sat_number_str = self.gps_segments[3 + sats]
            if sat_number_str:
                try:
                    sat_number = int(sat_number_str)
                    sats_used.append(sat_number)
                except ValueError:
                    return False
            else:
                break

        # PDOP,HDOP,VDOP
        try:
            pdop = float(self.gps_segments[15])
            hdop = float(self.gps_segments[16])
            vdop = float(self.gps_segments[17])
        except ValueError:
            return False

        # Update Object Data
        self.fix_type = fix_type

        # If Fix is GOOD, update fix timestamp
        if fix_type > self.__NO_FIX:
            self.new_fix_time()

        self.satellites_used = sats_used
        self.hdop = hdop
        self.vdop = vdop
        self.pdop = pdop

        return True

    def gpgsv(self):
        """Parse Satellites in View (GSV) sentence. Updates number of SV Sentences,the number of the last SV sentence
        parsed, and data on each satellite present in the sentence"""
        try:
            num_sv_sentences = int(self.gps_segments[1])
            current_sv_sentence = int(self.gps_segments[2])
            sats_in_view = int(self.gps_segments[3])
        except ValueError:
            return False

        # Create a blank dict to store all the satellite data from this sentence in:
        # satellite PRN is key, tuple containing telemetry is value
        satellite_dict = dict()

        # Calculate  Number of Satelites to pull data for and thus how many segment positions to read
        if num_sv_sentences == current_sv_sentence:
            # Last sentence may have 1-4 satellites; 5 - 20 positions
            sat_segment_limit = (sats_in_view - ((num_sv_sentences - 1) * 4)) * 5
        else:
            sat_segment_limit = 20  # Non-last sentences have 4 satellites and thus read up to position 20

        # Try to recover data for up to 4 satellites in sentence
        for sats in range(4, sat_segment_limit, 4):

            # If a PRN is present, grab satellite data
            if self.gps_segments[sats]:
                try:
                    sat_id = int(self.gps_segments[sats])
                except (ValueError,IndexError):
                    return False

                try:  # elevation can be null (no value) when not tracking
                    elevation = int(self.gps_segments[sats+1])
                except (ValueError,IndexError):
                    elevation = None

                try:  # azimuth can be null (no value) when not tracking
                    azimuth = int(self.gps_segments[sats+2])
                except (ValueError,IndexError):
                    azimuth = None

                try:  # SNR can be null (no value) when not tracking
                    snr = int(self.gps_segments[sats+3])
                except (ValueError,IndexError):
                    snr = None
            # If no PRN is found, then the sentence has no more satellites to read
            else:
                break

            # Add Satellite Data to Sentence Dict
            satellite_dict[sat_id] = (elevation, azimuth, snr)

        # Update Object Data
        self.total_sv_sentences = num_sv_sentences
        self.last_sv_sentence = current_sv_sentence
        self.satellites_in_view = sats_in_view

        # For a new set of sentences, we either clear out the existing sat data or
        # update it as additional SV sentences are parsed
        if current_sv_sentence == 1:
            self.satellite_data = satellite_dict
        else:
            self.satellite_data.update(satellite_dict)

        return True

    ##########################################
    # Data Stream Handler Functions
    ##########################################

    def new_sentence(self):
        """Adjust Object Flags in Preparation for a New Sentence"""
        self.gps_segments = ['']
        self.active_segment = 0
        self.crc_xor = 0
        self.sentence_active = True
        self.process_crc = True
        self.char_count = 0

    def update(self, new_char):
        """Process a new input char and updates GPS object if necessary based on special characters ('$', ',', '*')
        Function builds a list of received string that are validate by CRC prior to parsing by the  appropriate
        sentence function. Returns sentence type on successful parse, None otherwise"""

        valid_sentence = False

        # Validate new_char is a printable char
        ascii_char = ord(new_char)

        if 10 <= ascii_char <= 126:
            self.char_count += 1

            # Write Character to log file if enabled
            if self.log_en:
                self.write_log(new_char)

            # Check if a new string is starting ($)
            if new_char == '$':
                self.new_sentence()
                return None

            elif self.sentence_active:

                # Check if sentence is ending (*)
                if new_char == '*':
                    self.process_crc = False
                    self.active_segment += 1
                    self.gps_segments.append('')
                    return None

                # Check if a section is ended (,), Create a new substring to feed
                # characters to
                elif new_char == ',':
                    self.active_segment += 1
                    self.gps_segments.append('')

                # Store All Other printable character and check CRC when ready
                else:
                    self.gps_segments[self.active_segment] += new_char

                    # When CRC input is disabled, sentence is nearly complete
                    if not self.process_crc:

                        if len(self.gps_segments[self.active_segment]) == 2:
                            try:
                                final_crc = int(self.gps_segments[self.active_segment], 16)
                                if self.crc_xor == final_crc:
                                    valid_sentence = True
                                else:
                                    self.crc_fails += 1
                            except ValueError:
                                pass  # CRC Value was deformed and could not have been correct

                # Update CRC
                if self.process_crc:
                    self.crc_xor ^= ascii_char

                # If a Valid Sentence Was received and it's a supported sentence, then parse it!!
                if valid_sentence:
                    self.clean_sentences += 1  # Increment clean sentences received
                    self.sentence_active = False  # Clear Active Processing Flag

                    if self.gps_segments[0] in self.supported_sentences:

                        # parse the Sentence Based on the message type, return True if parse is clean
                        if self.supported_sentences[self.gps_segments[0]](self):

                            # Let host know that the GPS object was updated by returning parsed sentence type
                            self.parsed_sentences += 1
                            return self.gps_segments[0]

                # Check that the sentence buffer isn't filling up with Garage waiting for the sentence to complete
                if self.char_count > self.SENTENCE_LIMIT:
                    self.sentence_active = False

        # Tell Host no new sentence was parsed
        return None

    def new_fix_time(self):
        """Updates a high resolution counter with current time when fix is updated. Currently only triggered from
        GGA, GSA and RMC sentences"""
        try:
            self.fix_time = utime.ticks_ms()
        except NameError:
            self.fix_time = time.time()

    #########################################
    # User Helper Functions
    # These functions make working with the GPS object data easier
    #########################################

    def satellite_data_updated(self):
        """
        Checks if the all the GSV sentences in a group have been read, making satellite data complete
        :return: boolean
        """
        if self.total_sv_sentences > 0 and self.total_sv_sentences == self.last_sv_sentence:
            return True
        else:
            return False

    def satellites_visible(self):
        """
        Returns a list of of the satellite PRNs currently visible to the receiver
        :return: list
        """
        return list(self.satellite_data.keys())

    def time_since_fix(self):
        """Returns number of millisecond since the last sentence with a valid fix was parsed. Returns 0 if
        no fix has been found"""

        # Test if a Fix has been found
        if self.fix_time == 0:
            return -1

        # Try calculating fix time using utime; if not running MicroPython
        # time.time() returns a floating point value in secs
        try:
            current = utime.ticks_diff(utime.ticks_ms(), self.fix_time)
        except NameError:
            current = (time.time() - self.fix_time) * 1000  # ms

        return current

    def compass_direction(self):
        """
        Determine a cardinal or inter-cardinal direction based on current course.
        :return: string
        """
        # Calculate the offset for a rotated compass
        if self.course >= 348.75:
            offset_course = 360 - self.course
        else:
            offset_course = self.course + 11.25

        # Each compass point is separated by 22.5 degrees, divide to find lookup value
        dir_index = floor(offset_course / 22.5)

        final_dir = self.__DIRECTIONS[dir_index]

        return final_dir

    def latitude_string(self):
        """
        Create a readable string of the current latitude data
        :return: string
        """
        if self.coord_format == 'dd':
            formatted_latitude = self.latitude
            lat_string = str(formatted_latitude[0]) + '° ' + str(self._latitude[2])
        elif self.coord_format == 'dms':
            formatted_latitude = self.latitude
            lat_string = str(formatted_latitude[0]) + '° ' + str(formatted_latitude[1]) + "' " + str(formatted_latitude[2]) + '" ' + str(formatted_latitude[3])
        else:
            lat_string = str(self._latitude[0]) + '° ' + str(self._latitude[1]) + "' " + str(self._latitude[2])
        return lat_string

    def longitude_string(self):
        """
        Create a readable string of the current longitude data
        :return: string
        """
        if self.coord_format == 'dd':
            formatted_longitude = self.longitude
            lon_string = str(formatted_longitude[0]) + '° ' + str(self._longitude[2])
        elif self.coord_format == 'dms':
            formatted_longitude = self.longitude
            lon_string = str(formatted_longitude[0]) + '° ' + str(formatted_longitude[1]) + "' " + str(formatted_longitude[2]) + '" ' + str(formatted_longitude[3])
        else:
            lon_string = str(self._longitude[0]) + '° ' + str(self._longitude[1]) + "' " + str(self._longitude[2])
        return lon_string

    def speed_string(self, unit='kph'):
        """
        Creates a readable string of the current speed data in one of three units
        :param unit: string of 'kph','mph, or 'knot'
        :return:
        """
        if unit == 'mph':
            speed_string = str(self.speed[1]) + ' mph'

        elif unit == 'knot':
            if self.speed[0] == 1:
                unit_str = ' knot'
            else:
                unit_str = ' knots'
            speed_string = str(self.speed[0]) + unit_str

        else:
            speed_string = str(self.speed[2]) + ' km/h'

        return speed_string

    def date_string(self, formatting='s_mdy', century='20'):
        """
        Creates a readable string of the current date.
        Can select between long format: Januray 1st, 2014
        or two short formats:
        11/01/2014 (MM/DD/YYYY)
        01/11/2014 (DD/MM/YYYY)
        :param formatting: string 's_mdy', 's_dmy', or 'long'
        :param century: int delineating the century the GPS data is from (19 for 19XX, 20 for 20XX)
        :return: date_string  string with long or short format date
        """

        # Long Format Januray 1st, 2014
        if formatting == 'long':
            # Retrieve Month string from private set
            month = self.__MONTHS[self.date[1] - 1]

            # Determine Date Suffix
            if self.date[0] in (1, 21, 31):
                suffix = 'st'
            elif self.date[0] in (2, 22):
                suffix = 'nd'
            elif self.date[0] == (3, 23):
                suffix = 'rd'
            else:
                suffix = 'th'

            day = str(self.date[0]) + suffix  # Create Day String

            year = century + str(self.date[2])  # Create Year String

            date_string = month + ' ' + day + ', ' + year  # Put it all together

        else:
            # Add leading zeros to day string if necessary
            if self.date[0] < 10:
                day = '0' + str(self.date[0])
            else:
                day = str(self.date[0])

            # Add leading zeros to month string if necessary
            if self.date[1] < 10:
                month = '0' + str(self.date[1])
            else:
                month = str(self.date[1])

            # Add leading zeros to year string if necessary
            if self.date[2] < 10:
                year = '0' + str(self.date[2])
            else:
                year = str(self.date[2])

            # Build final string based on desired formatting
            if formatting == 's_dmy':
                date_string = day + '/' + month + '/' + year

            else:  # Default date format
                date_string = month + '/' + day + '/' + year

        return date_string

    # All the currently supported NMEA sentences
    supported_sentences = {'GPRMC': gprmc, 'GLRMC': gprmc,
                           'GPGGA': gpgga, 'GLGGA': gpgga,
                           'GPVTG': gpvtg, 'GLVTG': gpvtg,
                           'GPGSA': gpgsa, 'GLGSA': gpgsa,
                           'GPGSV': gpgsv, 'GLGSV': gpgsv,
                           'GPGLL': gpgll, 'GLGLL': gpgll,
                           'GNGGA': gpgga, 'GNRMC': gprmc,
                           'GNVTG': gpvtg, 'GNGLL': gpgll,
                           'GNGSA': gpgsa,
                          }



# -*- coding: utf-8 -*-
# Copyright (c) 2019 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for Pmod_SWT
"""

import RPi.GPIO as GPIO
CAP = 'GPIO'
PHY = '1x6'

class PmodSWT:
    def __init__(self,DSPmod6):
        # GPIO setup
        self.port = DSPmod6
        self.SWT1 = self.port.pin1
        self.SWT2 = self.port.pin2
        self.SWT3 = self.port.pin3
        self.SWT4 = self.port.pin4
        GPIO.setmode(GPIO.BCM)  #
        GPIO.setup(self.SWT1,GPIO.IN)   #
        GPIO.setup(self.SWT2,GPIO.IN)   #
        GPIO.setup(self.SWT3,GPIO.IN)   #
        GPIO.setup(self.SWT4,GPIO.IN)   #

    def cleanup(self):
        GPIO.cleanup()

    def GetSwitchPin(self,SW):
        # returns phisical GPIO Pin assignment
        if SW == 1:
            return self.SWT1
        elif SW == 2:
            return self.SWT2
        elif SW == 3:
            return self.SWT3
        elif SW == 4:
            return self.SWT4
        else:
            return None
    def GetStatus(self,SW):
        # returns switch status
        switchPin = self.GetSwitchPin(SW)
        return GPIO.input(switchPin)
    def AllOn(self):
        # returns "True" if all pins are ON
        return GPIO.input(GetSwitchPin(1)) and GPIO.input(GetSwitchPin(2)) and GPIO.input(GetSwitchPin(3)) and GPIO.input(GetSwitchPin(4))
    def AllOff(self):
        # returns "True" if all pins are OFF
        return (not GPIO.input(GetSwitchPin(1))) and (not GPIO.input(GetSwitchPin(2))) and (not GPIO.input(GetSwitchPin(3))) and (not GPIO.input(GetSwitchPin(4)))



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
A Python library for use with a Raspberry Pi, Pmod HAT and selected Pmods
"""

__version__ = '0.2.0'



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Manages Pmod HAT port resources, enforcing correct usage and 
avoiding conflicts.
"""

from __future__ import absolute_import
from DesignSpark.Pmod import TC1
from DesignSpark.Pmod import AD1
from DesignSpark.Pmod import MIC3
from DesignSpark.Pmod import HB3
from DesignSpark.Pmod import ISNS20
from DesignSpark.Pmod import OLEDrgb
from DesignSpark.Pmod import Error
from DesignSpark.Pmod import SWT
from DesignSpark.Pmod import ACL2
from DesignSpark.Pmod import GPS
from DesignSpark.Pmod import LS1
from DesignSpark.Pmod import KYPD


# Physical pin map of Pmod to BCM I/O

JA1 = 8  #SPI_CE0
JA2 = 10 #SPI0_MOSI
JA3 = 9  #SPI0_MISO
JA4 = 11 #SPI0_CLK
# Gnd JA5
# Vcc JA6 
JA7 = 19  #PCM_FS/PWM1
JA8 = 21  #PCM_DOUT/GPCLK1
JA9 = 20  #PCM_DIN/GPCLK0
JA10 = 18 #PCM_CLK/PWM0
# Gnd JA11
# Vcc JA12

JB1 = 7  #SPI_CE1
JB2 = 10 #SPI0_MOSI
JB3 = 9  #SPI0_MISO
JB4 = 11 #SPI_CLK
# Gnd JB5
# Vcc JB6 
JB7 = 26
JB8 = 13 #PWM1
JB9 = 3  #SCL1
JB10 = 2 #SDA1
# Gnd JB11
# Vcc JB12

JC1 = 16  #CTS0
JC2 = 14 #TXD0
JC3 = 15  #RXD0
JC4 = 17 #RTS0
# Gnd  JB5
# Vcc JB6 
JC7 = 4  #GPCLK0
JC8 = 12 #PWM0
JC9 = 5  #GPCLK1
JC10 = 6 #GPCLK2
# Gnd JB11
# Vcc JB12

moduleDict = {
    'TC1': TC1,
    'AD1': AD1,
    'MIC3': MIC3,
    'HB3': HB3,
    'ISNS20': ISNS20,
    'OLEDrgb': OLEDrgb,
    'SWT': SWT,
    'ACL2': ACL2,
    'GPS': GPS,
    'LS1': LS1,
    'KYPD': KYPD
    }

capabilityDict = {
    'JAA': [JA1,JA2,JA3,JA4,True,False,False],
    'JAB': [JA7,JA8,JA9,JA10,False,False,False],
    'JBA': [JB1,JB2,JB3,JB4,True,False,False],
    'JBB': [JB7,JB8,JB9,JB10,False,True,False],
    'JCA': [JC1,JC2,JC3,JC4,False,False,True], # UART pins are active by default 
    'JCB': [JC7,JC8,JC9,JC10,False,False,False]
    }

portUseDict = {}

def __checkPhysical(module, port):
    return module.PHY == port.phy

def __checkCapability(module, port):
    if module.CAP == 'SPI' and port._spi == True:
        return True
    if module.CAP == 'I2C' and port._i2c == True:
        return True
    if module.CAP == 'UART' and port._uart == True:
        return True
    if module.CAP == 'GPIO':
        return True
    # No capability match found
    return False

def createPmod(moduleName, portName):
        if moduleName in moduleDict:
            module = moduleDict[moduleName]
        else:
            erma = moduleName+' is not a recognised module identifier'
            raise Error.incorrectModuleName(erma) 
    
        if portName == 'JA' or portName == 'JB' or portName == 'JC':
            port = DSPMod12(portName)
        else:
            if portName in capabilityDict:
                port = DSPMod6(portName)
            else:
                erma = portName+' is not a recognised port identifier'
                raise Error.incorrectPortName(erma)
        
        if __checkPhysical(module,port) == False:
            erma = 'chosen port '+portName+' is the wrong size'+moduleName+' is a Pmod'+module.PHY+' module' 
            raise Error.portCapabilitySupport(erma)
        
        if __checkCapability(module,port) == False:
            erma = 'chosen port '+portName+' does not support '+module.CAP
            raise Error.portCapabilitySupport(erma)

        if port.inUse():
            erma = 'chosen port '+portName+' is already in use with '+portUseDict[portName]
            raise Error.portInUse(erma)
        
        ## Usage conflicts
        """
        If SPI is used on JAA or JBB allow only other SPI modules to init
        If a non-SPI module is used on JAA or JBB do not allow any other module 
        """
        
        if 'JAA' in portUseDict:
            cap = moduleDict[portUseDict['JAA']].CAP
            if cap == 'SPI':
                if portName == 'JBA' or portName == 'JB':
                    if moduleDict[moduleName].CAP != 'SPI':
                        erma = 'when port JAA is using an SPI module, port '+portName+' must also only use SPI'
                        raise Error.portCapabilityConflict(erma)
            else:       
                if portName == 'JBA' or portName == 'JB':
                    erma = 'when port JAA is not using an SPI module, port JBA is unavailable'
                    raise Error.portCapabilityConflict(erma) 


        if 'JBA' in portUseDict:
            cap = moduleDict[portUseDict['JBA']].CAP
            if cap == 'SPI':
                if portName == 'JAA' or portName == 'JA':
                    if moduleDict[moduleName].CAP != 'SPI':
                        erma = 'when port JBA is using an SPI module port '+portName+' must also only use SPI'
                        raise Error.portCapabilityConflict(erma)
            else:       
                if portName == 'JAA' or portName == 'JA':
                    erma = 'when port JBA is not using an SPI module, port JAA is unavailable'
                    raise Error.portCapabilityConflict(erma) 
                    
        
        ## All qualifiers passed
        port.setUseModule(moduleName)

        if moduleName == 'HB3':
            return HB3.PmodHB3(port)
        
        if moduleName == 'AD1':
            return AD1.PmodAD1(port)   
        
        if moduleName == 'ISNS20':
            return ISNS20.PmodISNS20(port)
        
        if moduleName == 'MIC3':
            return MIC3.PmodMIC3(port)
        
        if moduleName == 'TC1':
            return TC1.PmodTC1(port)
        
        if moduleName == 'OLEDrgb':
            return OLEDrgb.PmodOLEDrgb(port)

        if moduleName == 'SWT':
            return SWT.PmodSWT(port)
        
        if moduleName == 'ACL2':
            return ACL2.PmodACL2(port)

        if moduleName == 'GPS':
            return GPS.PmodGPS(port)

        if moduleName == 'LS1':
            return LS1.PmodLS1(port)

        if moduleName == 'KYPD':
            return KYPD.PmodKYPD(port)

class DSPMod6:
    
    def __init__(self, _portName):
        self.portName = _portName
        cap = capabilityDict[self.portName]
        self.pin1 = cap[0]
        self.pin2 = cap[1]
        self.pin3 = cap[2]
        self.pin4 = cap[3]
        self._spi = cap[4] 
        self._i2c = cap[5]
        self._uart = cap[6]
        self.phy = '1x6'
         
    def setUseModule(self,moduleName):
        portUseDict[self.portName] = moduleName
        
    def inUse(self):
        return self.portName in portUseDict
    
class DSPMod12:
     
    def __init__ (self, _portName):
        
        self.dspMod6A = DSPMod6(_portName + 'A')
        self.dspMod6B = DSPMod6(_portName + 'B')
        
        self.pin1 = self.dspMod6A.pin1
        self.pin2 = self.dspMod6A.pin2
        self.pin3 = self.dspMod6A.pin3
        self.pin4 = self.dspMod6A.pin4
        self.pin7 = self.dspMod6B.pin1
        self.pin8 = self.dspMod6B.pin2
        self.pin9 = self.dspMod6B.pin3
        self.pin10 = self.dspMod6B.pin4
        
        self._spi = self.dspMod6A._spi # All SPI is on the upper 'a' Pmod port
        self._i2c = self.dspMod6B._i2c # Only I2C is on the lower 'b' Pmod port
        self._uart = self.dspMod6A._uart # Only UART is on the upper 'a' Pmod port
        self.phy = '2x6'
        
    def setUseModule(self, moduleName):
        self.dspMod6A.setUseModule(moduleName)
        self.dspMod6B.setUseModule(moduleName)
         
    def inUse(self):
        return self.dspMod6A.inUse() and self.dspMod6B.inUse()
        



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for PmodTC1 module (MAX31855).
"""

import spidev
CAP = 'SPI'
PHY = '1x6'

class PmodTC1:
    
    def __init__(self, DSPMod6):
        
        self.port = DSPMod6
        
        self.cs = self.port.pin1
        self.nc = self.port.pin2
        self.miso = self.port.pin3
        self.sclk = self.port.pin4
        
        self.spi = spidev.SpiDev()
        self.__startSPI()
       
    def __startSPI(self):
        if self.cs == 7: #CE1
            self.spi.open(0,1)
        elif self.cs == 8: #CE0
            self.spi.open(0,0)
        else:
            #throw exception here
            return
        self.spi.max_speed_hz = 400000
        self.spi.mode = 0b00
        
    def __stopSPI(self):
        self.spi.close()   
    
    def __spiRead32(self):
            resp = self.spi.xfer2([0x00, 0x00, 0x00, 0x00])
            w = 0
            for i in range(3): 
                w |= resp[i]
                if i < 3:
                    w <<= 8
            return w
              
    def readCelcius(self):
            w = self.__spiRead32()
            #is error?
            if w & 0x7:
                return float('nan')
            #is negative ?
            if w & 0x80000000:
                w = 0xFFFFC000 | (( w >> 18) & 0x00003FFFF)
            else:
            #its standard    
                w >>= 18

            cent = float(w)
            cent *= 0.25
            return cent
                                  
    def readFarenheit(self):
            f = self.readCelcius()
            f *= 9.0
            f /= 5.0                      
            f += 32
            return f
    
    def readInternal(self):
            w = self.__spiRead32()
            w >> 4
            w &= 0x7FF
             
            if w & 0x800:
            # is negative    
                w = 0x0F800 | (w & 0x7FF)
            internal = float(w)    
            internal  *= 0.0625  
            
            return internal
        
    def readError():
        w = self.__spiRead32()
        return w & 0x7
    
    def cleanup(self):
        self.__stopSPI()
        
    



# -*- coding: utf-8 -*-
# Copyright (c) 2017, 2020 RS Components Ltd
# SPDX-License-Identifier: MIT License

from setuptools import setup

setup(
    name = 'DesignSpark.Pmod',
    namespace_packages=['DesignSpark'],
    packages = ['DesignSpark.Pmod'],
    version = '0.3.0',
    description = 'Raspberry Pi Pmod HAT support library',
    author = 'RS Components',
    author_email = 'maint@abopen.com',
    url = 'https://github.com/designsparkrs/DesignSpark.Pmod',
    download_url = 'https://github.com/designsparkrs/DesignSpark.Pmod/archive/0.3.0.tar.gz',
    install_requires = ['future','RPI.GPIO','spidev','luma.oled','luma.core','pyserial'],
    license = 'MIT License',
    long_description = open('README.rst').read(),
    keywords = 'raspberry pi raspi pmod digilent designspark spi pwm adc oled hbridge gps',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Education',
        'Topic :: System :: Hardware',
        'Topic :: System :: Hardware :: Hardware Drivers'
    ]
)



"""
Interface for Pmod_KYPD
"""

import RPi.GPIO as GPIO

CAP = 'GPIO'
PHY = '2x6'

ROWNUM = 4; # 4 lines
COLNUM = 4; # 4 columms

# Declaration of the key of the keypad
DefaultKeyMap=[['1','2','3','A'],['4','5','6','B'],['7','8','9','C'],['0','F','E','D']]
keyMap=[['0','0','0','0'],['0','0','0','0'],['0','0','0','0'],['0','0','0','0']]


class PmodKYPD:
    def __init__(self,DSPmod12):
        # Assignement of pin for the keypad
        self.port = DSPmod12
        self.rowPins=[0,0,0,0]
        self.colPins=[0,0,0,0]
        self.colPins[0] = self.port.pin4
        self.colPins[1] = self.port.pin3
        self.colPins[2] = self.port.pin2
        self.colPins[3] = self.port.pin1
        self.rowPins[0] = self.port.pin10
        self.rowPins[1] = self.port.pin9
        self.rowPins[2] = self.port.pin8
        self.rowPins[3] = self.port.pin7
        GPIO.setmode(GPIO.BCM)  #
        for i in range(COLNUM):
            GPIO.setup(self.colPins[i],GPIO.OUT) #
        for i in range(ROWNUM):
            GPIO.setup(self.rowPins[i],GPIO.IN)  #
        self.setKeyMapDefault() # Default key map

    def cleanup(self):
        GPIO.cleanup()

    def setKeyMapDefault(self):
        for i in range(COLNUM):
            for j in range(ROWNUM):
                keyMap[i][j] = DefaultKeyMap[i][j]

    def setKeyMap(self,UsrKeyMap):
        # Set User KeyMap
        for i in range(COLNUM):
            for j in range(ROWNUM):
                keyMap[i][j] = UsrKeyMap[i][j]

    def getKey(self):
        # initialize col to be high
        for i in range(COLNUM):
            GPIO.output(self.colPins[i], GPIO.HIGH)
        for i in range(COLNUM):
            # start to scan colPins[i]
            GPIO.output(self.colPins[i],GPIO.LOW)
            for j in range(ROWNUM):
                # row[j]
                # print(i,j)
                if(GPIO.input(self.rowPins[j])== 0):
                    return keyMap[j][i]
                # stop scanning colPins[i]
            GPIO.output(self.colPins[i],GPIO.HIGH)

    def getColRow(self):
        # initialize col to be high
        for i in range(COLNUM):
            GPIO.output(self.colPins[i], GPIO.HIGH)
        for i in range(COLNUM):
            # start to scan colPins[i]
            GPIO.output(self.colPins[i],GPIO.LOW)
            for j in range(ROWNUM):
                # row[j]
                if GPIO.input(self.rowPins[j])== 0:
                    return j<<2|i
                # stop scanning colPins[i]
            GPIO.output(self.colPins[i],GPIO.HIGH)
            
    def getKeyMap(self):
        return keyMap



# -*- coding: utf-8 -*-
# Copyright (c) 2017 RS Components Ltd
# SPDX-License-Identifier: MIT License

"""
Interface for PmodMIC3 (ADCS7476 + Knowles Acoustics SPA2410LR5H-B).
"""

import spidev
CAP = 'SPI'
PHY = '1x6'


class PmodMIC3:
    MIC3_NO_BITS = 12
    dReference = 3.3
    
    def __init__(self, DSPMod6):
        self.port = DSPMod6
        
        self.cs = self.port.pin1
        self.nc = self.port.pin2
        self.miso = self.port.pin3
        self.sclk = self.port.pin4
        
        self.spi = spidev.SpiDev()
        self.__startSPI()
        
    def __startSPI(self):
        if self.cs == 7: #CE1
            self.spi.open(0,1)
        elif self.cs == 8: #CE0
            self.spi.open(0,0)
        else:
            #throw exception here
            return
        self.spi.max_speed_hz = 100000
        self.spi.mode = 0b00
        
    def __stopSPI(self):
        self.spi.close()
    
    def __spiRead16(self):
        resp = self.spi.xfer2([0x00, 0x00])
        w = resp[0] 
        w <<= 8
        w |= resp[1]
        return w
    
    def readIntegerValue(self):
        return self.__spiRead16()
        
    def readPhysicalValue(self):
        
        wIntVal = self.readIntegerValue()
        dVal = wIntVal* (self.dReference/((1<<self.MIC3_NO_BITS)-1))
        return dVal
        
    def cleanup(self):
        self.__stopSPI()
       
